create or replace and compile java source named itstar_filefilter as
package itstar.utils;

import java.io.File;
import java.io.FilenameFilter;
/**
 * File Filter to search for regular expressions
 * 
 * @author Stefan Armbruster
 * @version 1.0
 * 
 * 
 */
public class FileFilter implements FilenameFilter {
  private String regExPattern;

  public FileFilter(String pattern) {
    setPattern(pattern);
  }

  public boolean accept(File dir, String name) {
    return name.matches(this.regExPattern);
  }

  public void setPattern(String pattern) {
    this.regExPattern = pattern;
  }

}
/
create or replace and compile java source named itstar_fileutils as
package itstar.utils;

import java.io.File;
import java.io.FilenameFilter;

/**
 * Utils for File-IO
 * 
 * @author Stefan Armbruster
 * 
 */
public class FileUtils {

  public static File[] getDirectoryList(String directoryPath,
      String filterPattern) throws Exception {

    File directory;
    File[] fileList = null;
    FilenameFilter filter;

    try {
      directory = new File(directoryPath);
      // if a regular expression is given, filter it
      if (filterPattern != null) {
        filter = new FileFilter(filterPattern);
        fileList = directory.listFiles(filter);
      } else {
        fileList = directory.listFiles();
      }

    } catch (Exception ex) {
      throw new Exception(ex.getLocalizedMessage());
    }

    return (fileList);
  }

  public static File[] getDirectoryList(String directoryPath)
      throws Exception {
    return (getDirectoryList(directoryPath, null));
  }
}
/
create or replace and compile java source named itstar_datetime as
package itstar.utils;

import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Date-Time handling
 * 
 * @author Stefan Armbruster
 * 
 */
public class DateTime {
  public static final String DATE_FORMAT_NOW = "dd-MM-yyyy HH:mm:ss";
  public static final String DATE_FORMAT_FILENAME = "yyyyMMddHHmmss";
  public static final String DATA_FORMAT_XSD_DATETIME = "yyyy-MM-dd'T'HH:mm:ss";
  public static final String DATA_FORMAT_XSD_DATE = "yyyy-MM-dd";
  public static final String DATE_FORMAT_GER = "dd.MM.yyyy";

  /**
   * converts a date into a string, using format masks
   * 
   * @param dateTime
   * @param dateTimeFormat
   * @return String mit dem Datum/Uhrzeit
   */
  public static String getStringFromDate(Date dateTime, String dateTimeFormat) {
    SimpleDateFormat sdf = new SimpleDateFormat(dateTimeFormat);
    // return sdf.format(cal.getTime());
		return sdf.format(dateTime);
	}
}
/
create or replace and compile java source named itstar_owfileutils as
package itstar.wrapper;

import java.io.*;
import java.sql.*;
import itstar.utils.FileUtils;
import itstar.utils.DateTime;


/* Class: OWFileUtils
 * Bedeutung: Wrapper Klasse für den Zugriff auf MDKBW-File-Utils
 * Autor: Stefan Armbruster, 12.2014
 */
public class OWFileUtils {

  public static void OWgetDirectoryList(String directoryPath,
      String filterPattern, Integer sessionId) throws Exception {

    File[] fileList = null;
    String fileType;
    String fileName;
    Long fileSize;
    Date tmpDate;
    String fileLastModified;
    String accessType = "DIRECTORY_LIST";

    fileList = FileUtils.getDirectoryList(directoryPath, filterPattern);

    for (int i = 0; i < fileList.length; i++) {
      if (fileList[i].isDirectory()) {
        fileType = "DIR";
      } else {
        fileType = "FILE";
      }
      fileName = fileList[i].getName();
      fileSize = fileList[i].length();
			tmpDate = new Date(fileList[i].lastModified());
			fileLastModified = DateTime.getStringFromDate(tmpDate, DateTime.DATE_FORMAT_FILENAME);
      
      /*
       * SQRJ: Insert the results into the stack table
       */ 
        #sql { insert into SYS_JAVA2PLSQL_PARA_TB (session_id, access_type, field1, field2, field3, field4, field5) values 
                                                  (:sessionId, :accessType, :directoryPath, :fileName, :fileType, :fileSize, :fileLastModified) 
        };
			 /* 
			 * SQRJ: End
			 */
		}
	}
}
/
CREATE or replace TYPE o_rec_dir_list_entry AS OBJECT (
 fileName varchar2(500),
 fileType varchar2(50),
 fileSize number,
 fileDate timestamp)
/
CREATE or replace TYPE o_tab_dir_list AS TABLE OF o_rec_dir_list_entry
/
create table SYS_JAVA2PLSQL_PARA_TB
(
  session_id  NUMBER not null,
  access_type VARCHAR2(50),
  field1      VARCHAR2(1000),
  field2      VARCHAR2(1000),
  field3      VARCHAR2(1000),
  field4      VARCHAR2(1000),
  field5      VARCHAR2(1000),
  field6      VARCHAR2(1000),
  field7      VARCHAR2(1000),
  field8      VARCHAR2(1000),
  field9      VARCHAR2(1000),
  field10     VARCHAR2(1000),
  field11     VARCHAR2(1000),
  field12     VARCHAR2(1000),
  field13     VARCHAR2(1000),
  field14     VARCHAR2(1000),
  field15     VARCHAR2(1000),
  field16     VARCHAR2(1000),
  field17     VARCHAR2(1000),
  field18     VARCHAR2(1000),
  field19     VARCHAR2(1000),
  field20     VARCHAR2(1000)
)
/
create or replace package JAVA_WRAPPER is

  function get_dir_list(p_directory in varchar2, p_filter in varchar2)
    return o_tab_dir_list;

  function get_dir_list_pipe(p_directory in varchar2) return o_tab_dir_list
    pipelined;
end JAVA_WRAPPER;
/
create or replace package body JAVA_WRAPPER is
  -------------------------------------------------------------------------------------------------------------------------------
  -- Java Wrapper Funktionen / nur innerhalb dieser Package verwendbar
  -------------------------------------------------------------------------------------------------------------------------------
  procedure JWgetDirectoryList(p_directory  in varchar2,
                               p_filter     in varchar2,
                               p_session_id in number) as
    language java name 'itstar.wrapper.OWFileUtils.OWgetDirectoryList(java.lang.String, java.lang.String, java.lang.Integer )';

  -------------------------------------------------------------------------------------------------------------------------------
  -- Java Gateway Funktionen / von "Außen" aufrufbar
  -------------------------------------------------------------------------------------------------------------------------------
  function get_dir_list(p_directory in varchar2, p_filter in varchar2)
    return o_tab_dir_list is
    PRAGMA AUTONOMOUS_TRANSACTION;
  
    l_session_id number;
    l_rec_dir    o_rec_dir_list_entry;
    l_tab_dir    o_tab_dir_list;
    l_idx        number := 0;
  
    cursor c_data(ci_session_id in number) is
      select *
        from SYS_JAVA2PLSQL_PARA_TB a
       where a.session_id = ci_session_id;
  
  begin
  
    l_rec_dir := o_rec_dir_list_entry(null, null, null, null);
    l_tab_dir := o_tab_dir_list();
  
    l_session_id := common_seq.nextval;
  
    -- Aufruf JavaWrapper Funktion
    JWgetDirectoryList(p_directory, p_filter, l_session_id);
  
    -- Verzeichnis-Inhalte aus der Parameter-Tabelle in die Collection übertragen
    for c1 in c_data(l_session_id) loop
      l_idx              := l_idx + 1;
      l_rec_dir.fileName := c1.field2;
      l_rec_dir.fileType := c1.field3;
      l_rec_dir.fileSize := to_number(c1.field4);
      l_rec_dir.fileDate := to_timestamp(c1.field5, 'YYYYMMDDHH24MISS');
      --l_tab_dir(l_idx) := l_rec_dir;
      l_tab_dir.extend;
      l_tab_dir(l_tab_dir.last) := l_rec_dir;
    end loop;
  
    -- Alle Daten die in dieser Session in die Parameter-Tabelle übertragen wurden,
    -- können nun wieder gelöscht werden. Statt delete deshalb einfach ein rollback
    rollback;
  
    return(l_tab_dir);
  
  exception
    when others then
      rollback;
      raise_application_error(-20001, 'File not found');
    
  end;

  -- Pipeline Function, um die Ergebnisse auch in einem Select verwenden zu können
  function get_dir_list_pipe(p_directory in varchar2) return o_tab_dir_list
    pipelined is
  
    l_rec_dir o_rec_dir_list_entry;
    l_tab_dir o_tab_dir_list;
  
  begin
  
    l_tab_dir := get_dir_list(p_directory, null);
  
    for i in 1 .. l_tab_dir.count loop
      l_rec_dir := l_tab_dir(i);
      pipe row(o_rec_dir_list_entry(l_rec_dir.fileName,
                                    l_rec_dir.fileType,
                                    l_rec_dir.fileSize,
                                    l_rec_dir.fileDate));
    
    end loop;
  
    return;
  end;
begin
  null;
end;
/
